import pytest

from huscy.attributes import services
from huscy.attributes.models import AttributeSchema

pytestmark = pytest.mark.django_db


def test_create_new_schema_if_none_exists():
    assert not AttributeSchema.objects.exists()

    attribute_schema = services.get_attribute_schema()

    assert 1 == AttributeSchema.objects.count()
    attribute_schema.schema == {'type': 'object', 'properties': {}}


def test_get_attribute_schema_without_version_number(attribute_schema_v4, schema_v4):
    assert schema_v4 == services.get_attribute_schema().schema


def test_get_attribute_schema_with_version_number(django_db_reset_sequences,
                                                  attribute_schema_v4, schema_v2):
    assert schema_v2 == services.get_attribute_schema(version=2).schema


def test_get_attribute_schema_with_invalid_version_number(django_db_reset_sequences,
                                                          attribute_schema_v4):
    with pytest.raises(AttributeSchema.DoesNotExist):
        services.get_attribute_schema(version=7)



import pytest

from huscy.attributes.services import get_or_create_attribute_set

pytestmark = pytest.mark.django_db


def test_return_existing_attribute_set(attribute_set, subject):
    result = get_or_create_attribute_set(subject)

    assert result == attribute_set


def test_return_new_created_attribute_set(django_db_reset_sequences, attribute_schema_v4,
                                          pseudonym):
    attribute_set = get_or_create_attribute_set(pseudonym.subject)

    assert attribute_set.pseudonym == '1234'
    assert attribute_set.attributes == {}
    assert attribute_set.attribute_schema == attribute_schema_v4



from django.contrib import admin

from huscy.attributes import models
from huscy.attributes.services import _create_attribute_category_permissions


class AttributeSchemaAdmin(admin.ModelAdmin):
    list_display = 'id', 'schema'
    list_display_links = None

    def save_model(self, request, attribute_schema, form, change):
        super().save_model(request, attribute_schema, form, change)
        _create_attribute_category_permissions(attribute_schema.schema)

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class AttributeSetAdmin(admin.ModelAdmin):
    list_display = 'pseudonym', '_attribute_schema', 'attributes'

    def _attribute_schema(self, attribute_set):
        return attribute_set.attribute_schema.id

    def has_add_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


admin.site.register(models.AttributeSchema, AttributeSchemaAdmin)
admin.site.register(models.AttributeSet, AttributeSetAdmin)



import pytest
from model_bakery import baker

from django.contrib.auth.models import Permission
from rest_framework.reverse import reverse
from rest_framework.status import HTTP_200_OK, HTTP_403_FORBIDDEN

from huscy.attributes.serializer import AttributeSchemaSerializer


def test_admin_user_can_retrieve_attribute_set(admin_client, attribute_set, subject):
    response = retrieve_attribute_set(admin_client, subject)

    assert HTTP_200_OK == response.status_code


def test_admin_user_can_update_attribute_set(admin_client, attribute_set, subject):
    response = update_attribute_set(admin_client, attribute_set, subject)

    assert HTTP_200_OK == response.status_code


def test_user_with_permission_can_retrieve_attribute_set(client, user, attribute_set, subject):
    view_permission = Permission.objects.get(codename='view_attributeset')
    user.user_permissions.add(view_permission)

    response = retrieve_attribute_set(client, subject)

    assert HTTP_200_OK == response.status_code


def test_user_with_permission_can_update_attribute_set(client, user, attribute_set, subject):
    change_subject_permission = Permission.objects.get(codename='change_subject')
    change_attributeset_permission = Permission.objects.get(codename='change_attributeset')
    user.user_permissions.add(change_subject_permission, change_attributeset_permission)

    response = update_attribute_set(client, attribute_set, subject)

    assert HTTP_200_OK == response.status_code


@pytest.mark.parametrize('permission', ['change_subject', 'change_attributeset'])
def test_user_with_either_change_subject_or_change_attributeset_perms_cannot_update_attribute_set(
        client, user, attribute_set, subject, permission):
    user.user_permissions.add(Permission.objects.get(codename=permission))

    response = update_attribute_set(client, attribute_set, subject)

    assert HTTP_403_FORBIDDEN == response.status_code


def test_user_without_permission_cannot_retrieve_attribute_set(client, attribute_set, subject):
    response = retrieve_attribute_set(client, subject)

    assert HTTP_403_FORBIDDEN == response.status_code


def test_user_without_permission_cannot_update_attribute_set(client, attribute_set, subject):
    response = update_attribute_set(client, attribute_set, subject)

    assert HTTP_403_FORBIDDEN == response.status_code


def test_anonymous_user_cannot_retrieve_attribute_set(anonymous_client, attribute_set, subject):
    response = retrieve_attribute_set(anonymous_client, subject)

    assert HTTP_403_FORBIDDEN == response.status_code


def test_anonymous_user_can_update_attribute_set(anonymous_client, attribute_set, subject):
    response = update_attribute_set(anonymous_client, attribute_set, subject)

    assert HTTP_403_FORBIDDEN == response.status_code


@pytest.mark.skip
def test_create_attribute_set_if_it_does_not_exist(django_db_reset_sequences, client, user,
                                                   attribute_schema_v2):
    view_permission = Permission.objects.get(codename='view_attributeset')
    user.user_permissions.add(view_permission)

    subject = baker.make('subjects.Subject')

    response = retrieve_attribute_set(client, subject)

    expected = {
        'attributes': {},
        'attribute_schema': AttributeSchemaSerializer(attribute_schema_v2).data
    }

    assert HTTP_200_OK == response.status_code
    assert expected == response.json()


def test_update_categorized_attribute_without_permission(client, user,
                                                         attribute_set_with_categories, subject):
    user.user_permissions.add(
        Permission.objects.get(codename='change_subject'),
        Permission.objects.get(codename='change_attributeset'),
    )

    response = client.put(
        reverse('attributeset-detail', kwargs=dict(pk=subject.id)),
        data={
            'attribute_schema': attribute_set_with_categories.attribute_schema.pk,
            'attributes': {
                'category1': {'attribute12': 99}
            },
        },
        format='json',
    )

    assert HTTP_403_FORBIDDEN == response.status_code


def test_update_categorized_attribute_with_permission(client, user,
                                                      attribute_set_with_categories, subject):
    user.user_permissions.add(
        Permission.objects.get(codename='change_subject'),
        Permission.objects.get(codename='change_attributeset'),
        Permission.objects.get(codename='change_attribute_category_category1'),
    )

    response = client.put(
        reverse('attributeset-detail', kwargs=dict(pk=subject.id)),
        data={
            'attribute_schema': attribute_set_with_categories.attribute_schema.pk,
            'attributes': {
                'category1': {'attribute12': 99}
            },
        },
        format='json',
    )

    assert HTTP_200_OK == response.status_code


def retrieve_attribute_set(client, subject):
    return client.get(reverse('attributeset-detail', kwargs=dict(pk=subject.id)))


def update_attribute_set(client, attribute_set, subject):
    return client.put(
        reverse('attributeset-detail', kwargs=dict(pk=subject.id)),
        data={
            'attribute_schema': attribute_set.attribute_schema.pk,
            'attributes': {
                'attribute1': {},
                'attribute2': 'another string',
                'attribute3': 1.0
            },
        },
        format='json',
    )



import pytest

from django.contrib.admin.sites import AdminSite

from huscy.attributes.admin import AttributeSchemaAdmin
from huscy.attributes.models import AttributeSchema

pytestmark = pytest.mark.django_db


@pytest.fixture
def attribute_schema_admin():
    return AttributeSchemaAdmin(model=AttributeSchema, admin_site=AdminSite())


def test_has_change_permission(attribute_schema_admin):
    assert attribute_schema_admin.has_change_permission(request=None) is False


def test_has_delete_permission(attribute_schema_admin):
    assert attribute_schema_admin.has_delete_permission(request=None) is False



from setuptools import find_namespace_packages, setup

from huscy.attributes import __version__


setup(
    name='huscy.attributes',
    version=__version__,
    license='AGPLv3+',

    author='Alexander Tyapkov, Mathias Goldau, Stefan Bunde',
    author_email='tyapkov@gmail.com, goldau@cbs.mpg.de, stefanbunde+git@posteo.de',

    packages=find_namespace_packages(),

    install_requires=[
        'huscy.pseudonyms',
        'jsonschema>=3',
    ],
    extras_require={
        'development': ['psycopg2-binary'],
        'testing': ['tox', 'watchdog'],
    },

    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.0',
    ],
)
